import os
import sys
from pathlib import Path


def load_path():
    """
    向上查找 [config.py or pyproject.toml] 所在目录，并将其添加到 sys.path 中。
    """
    current_dir = Path(sys.modules.get("__main__").__file__).resolve().parent
    root_dir = None
    for parent in [current_dir, *current_dir.parents]:
        if (parent / "config.py").exists() or (parent / "pyproject.toml").exists():
            root_dir = parent
            break
    if root_dir is not None:
        root_path = str(root_dir)
        if sys.path[0] != root_path:
            sys.path.insert(0, root_path)
            print(f"[load_path] 添加路径到 sys.path: {root_path}")
    else:
        print("[load_path] 未找到 config.py 或 pyproject.toml，未修改 sys.path。")



def load_env(filepath='.env'):
    """
    从指定路径读取 .env 文件，并将变量加载到 os.environ 中。
    :param filepath: .env 文件路径，默认为当前目录下的 .env
    """
    if not os.path.isfile(filepath):
        return

    with open(filepath, encoding='utf-8') as file:
        for line in file:
            line = line.strip()
            if not line or line.startswith('#'):
                continue
            if '=' in line:
                key, value = line.split('=', 1)
                os.environ[key] = value.strip('"\' ')


import concurrent.futures
import time

def task(n):
    print(f"Task {n} starting")
    time.sleep(2)
    return n * n

# 1. 手动创建线程池
executor = concurrent.futures.ThreadPoolExecutor(max_workers=3)

# 2. 提交多个任务（返回 Future 对象）
futures = []
for i in range(100):
    future = executor.submit(task, i)
    futures.append(future)



time.sleep(10)
print('stop')


# 4. 手动关闭线程池（重要！）
executor.shutdown(wait=True)  # wait=True 表示等待所有任务完成后再关闭




import datetime
import json
import re
from dataclasses import dataclass, field
from typing import Union, List, Optional, Dict, Callable, TYPE_CHECKING

from loguru import logger
from parsel import Selector

from ._internal import _join_url
from .lazy import LazyLoader

if TYPE_CHECKING:
    import dateparser
    import pandas as pd
else:
    dateparser = LazyLoader("dateparser")  # type: ignore
    pd = LazyLoader("pandas")  # type: ignore


@dataclass
class AttachmentsAndImagesResult:
    """
    封装附件和图片的结果
    """
    file_urls: List[str] = field(default_factory=list)  # 文件链接列表
    file_names: List[str] = field(default_factory=list)  # 文件名称列表
    image_urls: List[str] = field(default_factory=list)  # 图片链接列表
    image_names: List[str] = field(default_factory=list)  # 图片名称列表


def get_column_list(text: str, xpath: str, str_add_head: str = '', str_add_tail: str = '', auto_wash: bool = True) -> \
        list[str]:
    if isinstance(text, str):
        selector = Selector(text=text)
        value_list = selector.xpath(xpath).getall()
        if auto_wash:
            value_new_list = []
            for value in value_list:
                value = value.replace(' ', '').replace('\r', '').replace('\n', '').replace('\xa0', '').replace('\t', '')
                value_new_list.append(str_add_head + value + str_add_tail)
            return list(value_new_list)
        else:
            return value_list
    else:
        return []


def get_column(text: str, xpath: str, str_add_head: str = '', str_add_tail: str = '', auto_wash: bool = True) -> str:
    if isinstance(text, str):
        selector = Selector(text=text)
        value_list = selector.xpath(xpath).getall()
        if auto_wash:
            result = []
            for value in value_list:
                value = value.replace(' ', '').replace('\r', '').replace('\n', '').replace('\xa0', '').replace('\t',
                                                                                                               '').replace(
                    '\u3000', '')
                result.append(value)
            result_result = str_add_head + ''.join(result) + str_add_tail

            return result_result
        else:
            return ''.join(value_list)


def get_content(text: str, xpath_expression: str, auto_space: bool = True) -> str:
    """
    获取正文内容，并忽略style、script标签
    :param text: HTML文本
    :param xpath_expression: XPath表达式
    :param auto_space: 是否自动去除空格（True：去除所有空格，False：保留单个空格）
    :return: 清理后的文本内容
    """
    if not text:
        return ''

    selector = Selector(text=text)

    # 移除style和script标签
    for style in selector.xpath(xpath_expression + '//style'):
        style.drop()
    for script in selector.xpath(xpath_expression + '//script'):
        script.drop()

    # 提取内容并拼接
    contents = selector.xpath(xpath_expression).xpath('string(.)').getall()
    content = ''.join(contents).strip() \
        .replace('\r', '') \
        .replace('\n', '') \
        .replace('\xa0', '') \
        .replace('\t', '') \
        .strip()

    if auto_space:
        content = content.replace(' ', '')
    else:
        content = re.sub(r' +', ' ', content)

    return content


def xpath_kv(selector: Union[Selector, str], item_xpath: str, key_xpath: str, value_xpath: str) -> list[
    tuple[str, str]]:
    if isinstance(selector, str):
        selector = Selector(text=selector)
    item_list = selector.xpath(item_xpath)
    result = []
    for item_selector in item_list:
        name = ''.join(item_selector.xpath(key_xpath).getall()).strip().replace('\n', ' ').replace('\t', ' ').replace(
            '\r', ' ').replace(' ', '').replace('\xa0', '')
        v = item_selector.xpath(value_xpath).get()
        value = v if v else ''
        item = (name, value)
        result.append(item)
    return result

class Att:
    file_extensions = ['.rar', '.zip', '.7z', '.tar', '.gz', '.docx', '.doc', '.xlsx', '.xls', '.pdf', '.txt', '.csv',
                       '.et', '.ceb','.wps']
    image_extensions = ['.jpg', '.jpeg', '.png', '.bmp']


    @staticmethod
    def _handel_att_json(item_list: list[tuple[str, str]], base_url) -> str:
        result = {}
        i = 0
        for name, href in item_list:
            href = _join_url(base_url, href)
            fj = f'{name}|{href}'
            if fj not in result.values():
                i += 1
                result[f'fj{i}'] = fj
        if not result:
            return ""
        else:
            return json.dumps(result, ensure_ascii=False, separators=(',', ':'))

    @staticmethod
    def get_attachments(text: Union[Selector, str], xpath_expression: str, file_ext: Union[list, str] = '', image=False, base_url='', handel=False,contains_text:Union[list[str],str]='') -> Union[list[tuple[str, str]], str]:
        file_extensions = Att.file_extensions.copy()
        image_extensions = Att.image_extensions.copy()
        if isinstance(text, str):
            text = Selector(text=text)
        if file_ext:
            if isinstance(file_ext, str):
                file_ext = [file_ext]
            file_extensions += file_ext
        if image:
            file_extensions += image_extensions
        # 添加大写后缀
        file_extensions += [ext.upper() for ext in file_extensions]
        pattern = re.compile(r'(?:' + '|'.join(re.escape(e) for e in file_extensions) + r')(?=[?]|$)')
        file_extensions_xpath = ' or '.join(
            f'contains(@href,"{ext}")'
            # f'substring(@href, string-length(@href) - {len(ext) - 1}) = "{ext}"'
            for ext in file_extensions
        )
        file_a_xpath = f'{xpath_expression}//a[{file_extensions_xpath}]'
        file_result = xpath_kv(text, file_a_xpath, 'string(.)', './@href')
        file_result = [
            (name, href) for name, href in file_result
            if href and pattern.search(href)
        ]
        if contains_text:
            if isinstance(contains_text, str):
                contains_text = [contains_text]
            contains_xpath = ' or '.join(
                f'contains(@href,"{text}")'
                for text in contains_text
            )
            file_a_contains_xpath = f'{xpath_expression}//a[{contains_xpath}]'
            contains_result = xpath_kv(text, file_a_contains_xpath, 'string(.)', './@href')
            if contains_result:
                file_result.extend(contains_result)
        if handel: return Att._handel_att_json(file_result, base_url)
        return file_result

    @staticmethod
    def get_images(text: Union[Selector, str], xpath_expression: str, image_ext: Union[list, str] = '', base_url='', handel=False) -> Union[list[tuple[str, str]], str]:
        image_extensions = Att.image_extensions.copy()
        if image_ext:
            if isinstance(image_ext, str):
                image_ext = [image_ext]
            image_extensions += image_ext
        image_extensions += [ext.upper() for ext in image_extensions]
        pattern = re.compile(r'(?:' + '|'.join(re.escape(e) for e in image_extensions) + r')(?=[?]|$)')
        image_extensions_xpath = ' or '.join(
            f'contains(@src,"{ext}")'
            # f'substring(@src, string-length(@src) - {len(ext) - 1}) = "{ext}"'
            for ext in image_extensions
        )
        image_img_xpath = f'{xpath_expression}//img[{image_extensions_xpath}]'
        image_result = xpath_kv(text, image_img_xpath, './@alt', './@src')
        image_result = [
            (name, href) for name, href in image_result
            if href and pattern.search(href)
        ]
        if handel: return Att._handel_att_json(image_result, base_url)

        return image_result

def get_attachments_and_images(text: str, xpath_expression: str, file_ext: Union[list, str] = '') -> AttachmentsAndImagesResult:
    """
    获取正文中的附件和图片链接及名称
    :param text: HTML文本
    :param xpath_expression: XPath表达式
    :param file_ext: 额外的文件后缀（可选）
    :return: AttachmentsAndImagesResult 对象，包含文件链接、文件名称、图片链接、图片名称
    """
    selector = Selector(text=text)
    file_urls = []
    file_names = []
    file_result = Att.get_attachments(selector, xpath_expression, file_ext)
    for file in file_result:
        file_names.append(file[0])
        file_urls.append(file[1])
    image_urls = []
    image_names = []
    image_result = Att.get_images(selector, xpath_expression, file_ext)
    for image in image_result:
        image_names.append(image[0])
        image_urls.append(image[1])
    return AttachmentsAndImagesResult(
        file_urls=file_urls,
        file_names=file_names,
        image_urls=image_urls,
        image_names=image_names
    )


def get_p_list(xpath: str, text: str) -> list[str]:
    """
    获取这一级所有 p 标签的 content
    :param xpath: XPath 表达式
    :param text: HTML 文本
    :return: 清理后的 p 标签内容列表
    """
    selector = Selector(text=text)
    p_tags = selector.xpath(xpath).getall()
    ps = []
    for p in p_tags:
        # 使用 Selector 解析每个 p 标签的内容
        p_content = Selector(text=p).xpath('string(.)').get().strip() \
            .replace(' ', '') \
            .replace('\r', '') \
            .replace('\n', '') \
            .replace('\xa0', '') \
            .replace('\t', '') \
            .replace(' ', '')
        ps.append(p_content)
    return ps


def get_p_value(p_list: list[str], *keys: str) -> Optional[str]:
    """
    从 p_list 中查找以任意一个 key 开头的内容，并返回匹配的值
    """
    for p in p_list:
        for key in keys:
            if re_match := re.search(f'^{key}(.*)', p):
                return re_match.group(1).strip()
    return None


def parse_date(date_str: str) -> Union[datetime.datetime, None]:
    """
    动态解析日期字符串(严格模式)
    :param date_str:
    :return:
    """
    if not date_str:
        return None

    return dateparser.parse(str(date_str), settings={'STRICT_PARSING': True})


def parse_table(
        trs: List[str],
        header_str: Optional[str] = None,
        skip_line: Optional[int] = None,
        nested: bool = False,
        column_mapper: Optional[Dict[str, str]] = None,
        multi_header: bool = False,
        skip_empty_rows: bool = True,
        cell_formatter: Optional[Callable[[str], str]] = None
) -> List[Dict[str, Union[str, List[Dict[str, str]]]]]:
    """
        解析 HTML 表格，支持嵌套表格、自定义列名映射、多表头行
        :param trs: 表格里的 tr 标签数组
        :param header_str: 判断表头的字符串
        :param skip_line: 跳过指定行数
        :param nested: 是否解析嵌套表格
        :param column_mapper: 自定义列名映射规则
        :param multi_header: 是否支持多行表头
        :param skip_empty_rows: 是否跳过空行
        :param cell_formatter: 自定义单元格内容格式化函数
        :return: 解析后的表格数据（字典列表）
        """

    if not trs:
        return []

    # 跳过指定行数
    if skip_line and skip_line > 0:
        trs = trs[skip_line:]

    # 检查表头下标
    header_index = -1
    for i, tr in enumerate(trs):
        if header_str and header_str in tr:
            header_index = i
            break
    if header_index == -1:
        return []

    # 获取表头
    header_trs = [trs[header_index]]
    if multi_header:
        # 如果支持多行表头，继续向上查找表头行
        for i in range(header_index - 1, -1, -1):
            if any(tag in trs[i] for tag in ['<th>', '<td>']):
                header_trs.insert(0, trs[i])
            else:
                break

    columns = []
    for header_tr in header_trs:
        header_selector = Selector(text=header_tr)
        header_tds = header_selector.xpath('//td | //th').getall()
        current_columns = [
            Selector(text=td).xpath('string(.)').get().strip()
            .replace(' ', '')
            .replace('\r', '')
            .replace('\n', '')
            .replace('\xa0', '')
            .replace('\t', '')
            for td in header_tds
        ]
        if not columns:
            columns = current_columns
        else:
            # 合并多行表头
            columns = [f"{col1}_{col2}" if col2 else col1 for col1, col2 in zip(columns, current_columns)]

    # 自定义列名映射
    if column_mapper:
        columns = [column_mapper.get(col, col) for col in columns]

    # 解析表格内容
    result = []
    for tr in trs[header_index + 1:]:
        content_selector = Selector(text=tr)
        content_tds = content_selector.xpath('//td').getall()

        if len(content_tds) != len(columns):  # 如果列数不匹配，跳过该行
            continue

        item = {}
        for i, td in enumerate(content_tds):
            td_selector = Selector(text=td)
            td_text = (td_selector.xpath('string(.)').get().strip()
                       .replace('\r', '')
                       .replace('\n', '')
                       .replace('\xa0', '')
                       .replace('\t', ''))

            # 自定义单元格内容格式化
            if cell_formatter:
                td_text = cell_formatter(td_text)

            # 解析嵌套表格
            if nested and '<table' in td:
                nested_tables = td_selector.xpath('//table').getall()
                if nested_tables:
                    td_text = parse_table(nested_tables, header_str=header_str, nested=True)

            item[columns[i]] = td_text

        # 跳过空行
        if skip_empty_rows and all(not value for value in item.values()):
            continue

        result.append(item)

    return result


def parse_excel(path, header_str, merge=False):
    try:
        p = pd.read_excel(path)
    except Exception as e:
        logger.error(e)
        return []
    # header_str = '案件名称'
    # 获取表头那一行
    header_index = -1

    if header_str in p.columns:
        columns = p.columns
        header_index = 0
    else:
        for i, v in enumerate(p.values):
            if header_str in v:
                header_index = i
                break
        columns = p.values[header_index]
    if header_index == -1:
        return []
    result = []
    for v in p.values[header_index + 1:]:
        item = {}
        for i, column in enumerate(columns):
            item[column] = v[i]
        result.append(item)

    return result


def parse_excel_v2(path, header_str: Union[list[str], str]):
    # 如果header_str是字符串，则调用parse_excel
    if not isinstance(header_str, list):
        return parse_excel(path, header_str)
    if len(header_str) < 2:
        return []

    try:
        df = pd.read_excel(path, header=None)
    except Exception as e:
        logger.error(e)
        return []

    # 查找最后一个表头的索引
    last_header = header_str[-1]
    last_index = next((i for i, row in enumerate(df.values) if last_header in row), -1)

    if last_index == -1:
        return []

    # 验证表头行是否包含header_str中的元素
    header_start_index = last_index - len(header_str) + 1
    if header_start_index < 0 or not all(any(header_str[j] in str(cell) for cell in df.iloc[idx]) for j, idx in
                                         enumerate(range(header_start_index, last_index + 1))):
        columns = df.iloc[last_index].tolist()
    else:
        # 前向填充表头行
        header_df = df.iloc[header_start_index:last_index + 1].ffill(axis=0)
        # 取最后一行作为真正的表头
        columns = header_df.iloc[-1].tolist()

    # 生成结果列表
    result = []
    for row in df.values[last_index + 1:]:
        item = {col: value for col, value in zip(columns, row)}
        result.append(item)

    return result



import atexit
import json
import os
import sys
import threading
import time
import uuid
import datetime
from pathlib import Path
import traceback
from typing import Optional
from urllib.parse import urlparse

import requests
from loguru import logger
import socket

from .core import load_env

SPM_CONFIG = os.getenv('SPM_CONFIG', Path("C:\\spm.env") if sys.platform == "win32" else Path("/spm.env"))
load_env(SPM_CONFIG)
LOG_PATH = os.getenv('LOG_PATH', Path("C:\\logs") if sys.platform == "win32" else Path("/logs"))
PY_MONITOR_ENDPOINT = os.getenv('MONITOR_ENDPOINT', 'http://127.0.0.1:14000/api/endpoint')
MAX_RETRY = 100  # 最大上报重试次数
MONITOR_INTERVAL = 10  # 上报间隔


class SpiderInfo:
    def __init__(self, name=None, group_name=None, domain=None, url=None, freq=None, metadata=None, user=None,
                 monitor=True, insert_table=None, task_type=0, wan_ip='0.0.0.0', log=True, show_debug=False,
                 monitor_endpoint=None):
        """
        SpiderInfo 爬虫基础信息监控
        :param name: 名称
        :param group_name: 分组路径 /A/B/C
        :param domain: 目标站点域名
        :param url: 目标站点url
        :param freq: 更新频率: 时更，日更，周更，月更，一次性，任务表
        :param metadata: 其他元数据
        :param user: 所属用户 [ENV:SERVER_USER]
        :param monitor: 是否开启上报数据
        :param insert_table: 插入主表名
        :param task_type: 任务类型 default 0
        :param wan_ip: 外网IP [ENV:WAN_IP]
        :param log: 是否开启本地日志存储
        :param show_debug: 显示调试信息
        :param monitor_endpoint: 上报端点 [ENV:MONITOR_ENDPOINT]
        """
        self.__id = 0
        # 进程ID
        self.pid = str(os.getpid())
        # 运行唯一ID
        self.uid = str(uuid.uuid4())
        # 父ID
        self.parent_uid = None
        # 运行名称
        self.name = name
        # 分组 以 / 分割 eg: '学术/维普/期刊'
        self.group_name = group_name
        # 网址
        self.url = url
        # 域名
        self.domain = domain if domain else (urlparse(url).netloc if self.url else self.url)
        # 更新频率
        self.freq = freq
        # 任务类型
        self.task_type = task_type
        # 其他元数据
        self.metadata: dict = metadata

        # 开始时间
        self.start_time = datetime.datetime.now()
        # 结束时间
        self.end_time = None
        # 运行时间
        self.run_time = None
        # 日志文件
        self.log_file = None
        # py文件
        self.file = None
        # py解释器
        self.interpreter = sys.executable
        # 运行状态 0.待启动 1.运行中 2.完成 -1.异常 -2.断连
        self.status = 1
        # 插入数量
        self.insert_count = 0
        # 插入表名
        self.insert_table = insert_table
        # 进度
        self.progress = 0
        # 总进度
        self.total_progress = 0
        # 异常类名
        self.exception = None
        # 异常堆栈
        self.exception_stack = None

        # 消息
        self.msg = ''
        # 数据
        self.data = ''
        # 服务器名称
        self.server = os.getenv("SERVER_NAME", '未知')
        self.server_location = int(os.getenv("SERVER_LOCATION", 0))
        self.user = user or os.getenv("SERVER_USER")
        self.lan_ip = '127.0.0.1'
        self.wan_ip = os.getenv("WAN_IP", wan_ip)

        # 是否开启日志输出
        self.__log = log
        # 上报端点
        self.__monitor_endpoint = monitor_endpoint or PY_MONITOR_ENDPOINT
        # 是否输出debug日志
        self.__show_debug = show_debug
        # 是否上报数据
        self.__is_monitor = monitor
        # 获取当前日期
        self.__today = datetime.date.today()
        # 线程锁 (锁:插入量,进度)
        self.__lock = threading.Lock()

        self.__monitor_thread: Optional[threading.Thread] = None
        # self.__stop_event = threading.Event()

        if not monitor:
            return

        # 添加全局异常拦截器
        sys.excepthook = self.__global_exception_handler
        # 注册退出函数
        atexit.register(self.__before_exit)

        self._logeer_id = None
        # 获取主模块
        main_module = sys.modules.get('__main__')
        self.file = main_module.__file__  # 设置主文件全路径
        if self.__log:
            self.__initialize_log_file()
        self.__init_ip()

        self.__last_state = {}

        if self.__is_monitor:
            self.start_monitor()

    def __initialize_log_file(self):
        log_file = os.getenv('LOG_FILE')
        if log_file:
            # 不使用logger写入日志文件
            self.log_file = log_file
            return
        # 基础日志目录
        base_log_path = LOG_PATH
        original_path = Path(self.file).resolve()
        stem_name = original_path.stem
        timestamp = self.start_time.strftime("%m%d-%H%M%S.%f")[:-3].replace('.', '')
        log_filename = f"{stem_name}-{timestamp}.log"

        # 获取脚本路径相对于根目录的结构
        rel_path = original_path.parent.relative_to(original_path.anchor)
        full_path = base_log_path.joinpath(rel_path).joinpath(log_filename)
        full_path.parent.mkdir(parents=True, exist_ok=True)
        self.log_file = str(full_path)

        if self.log_file:
            self._logeer_id = logger.add(self.log_file, rotation="50 MB",retention=1)

    def __global_exception_handler(self, exc_type, exc_value, exc_traceback):
        self.debug('全局异常捕获')
        stack_trace = ''.join(traceback.format_exception(exc_type, exc_value, exc_traceback))
        logger.error(f"发生异常: {exc_type.__name__}: {exc_value}\n堆栈信息:\n{stack_trace}")
        self.exception = f"{exc_type.__module__}.{exc_type.__name__}"
        self.exception_stack = stack_trace
        self.status = -1
        sys.__excepthook__(exc_type, exc_value, exc_traceback)

    def __before_exit(self):
        self.debug('程序退出')
        self.end_time = datetime.datetime.now()
        self.run_time = self.end_time - self.start_time
        if self.status != -1:
            self.status = 2
        logger.warning(f'运行时长: {self.run_time} 秒')
        if self.__is_monitor:
            self.report()
        logger.warning(self)

    def __init_ip(self):
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(("223.5.5.5", 80))
            self.lan_ip = s.getsockname()[0]
            s.close()
        except Exception:
            try:
                self.lan_ip = socket.gethostbyname(socket.gethostname())
            except:
                pass
        if self.wan_ip == '0.0.0.0':
            try:
                response = requests.get('https://ipinfo.io/json', timeout=5)
                self.wan_ip = response.json()['ip']
            except:
                pass

    def add_count(self, v: int = 1):
        with self.__lock:
            self.insert_count += v

    def add_progress(self, v: int = 1):
        with self.__lock:
            self.progress += v

    def debug(self, msg):
        if self.__show_debug:
            logger.warning(msg)

    def to_dict(self) -> dict:
        data = {}
        for k, v in self.__dict__.items():
            if k.startswith('_'):
                continue

            parts = k.split('_')
            k = parts[0] + ''.join(word.capitalize() for word in parts[1:])

            if isinstance(v, datetime.datetime):
                data[k] = v.strftime("%Y-%m-%d %H:%M:%S")
            elif isinstance(v, datetime.timedelta):
                data[k] = v.total_seconds()
            elif isinstance(v, dict) and v:
                data[k] = json.dumps(v, ensure_ascii=False)
            elif isinstance(v, int) or isinstance(v, float):
                data[k] = v
            else:
                value = str(v)
                data[k] = None if value == 'None' else value
        return data

    def __monitor(self):
        num = 0
        while self.__is_monitor:
            # 如果脚本运行超过一天到达第二天则重置
            if datetime.date.today() > self.__today:
                self.reset()
                result = self.report(False)
            else:
                result = self.report(True)

            if not result:
                num += 1
                if num >= MAX_RETRY:
                    logger.error(f'上传数据失败，超过最大重试次数 {MAX_RETRY} 次，终止上报')
                    break
            else:
                num = 0
            time.sleep(MONITOR_INTERVAL)

    def report(self, filter_data=True):
        """
        上报数据
        :param filter_data: 是否过滤数据 （只上报变化的数据）
        :return:
        """
        try:
            self.debug('上报数据')
            data = self.to_dict()
            if filter_data:
                # 只上报发生变化的数据
                send_data = {k: data[k] for k in data if self.__last_state.get(k) != data.get(k)}
            else:
                send_data = data
            send_data['uid'] = self.uid  # 唯一键
            self.debug(send_data)
            response: dict = requests.post(url=self.__monitor_endpoint, json=send_data, timeout=4).json()
            self.debug(response)

            # 返回这条数据的ID主键
            # 老接口返回的data直接就是id
            # 新接口返回的data是对象下的id

            # 通过code判断是否成功
            if response['code'] != 200:
                return False
            # self.__id = int(response['data']['id']) # TODO __id  暂时不设置
            # 更新上次上报数据的状态
            self.__last_state.update(data)
            return True
        except Exception as e:
            self.debug(f'上报失败: {e}')
            return False

    def start_monitor(self):
        self.debug('启动监控')
        self.__monitor_thread = threading.Thread(target=self.__monitor)
        self.__monitor_thread.setDaemon(True)
        self.__monitor_thread.start()

    def reset(self):
        # 重置
        now = datetime.datetime.now()
        data = self.to_dict()
        data['run_time'] = (now - self.start_time).total_seconds()
        data['end_time'] = now.strftime("%Y-%m-%d %H:%M:%S")
        data['status'] = 2
        success = False
        for _ in range(5):
            try:
                self.debug('上报数据reset')
                self.debug(data)
                response = requests.post(url=self.__monitor_endpoint, json=data, timeout=4)
                self.debug(response.text)
                success = True
                break
            except Exception as e:
                self.debug(f'上报失败: {e}')
                continue
        if success:
            self.parent_uid = self.uid
            self.uid = str(uuid.uuid4())
            self.start_time = now
            self.insert_count = 0
            self.__today = now.date()

    def close(self):
        atexit.unregister(self.__before_exit)
        sys.excepthook = sys.__excepthook__
        self.__before_exit()
        self.__is_monitor = False

    def __repr__(self):
        return (
            f"SpiderInfo(pid={self.pid}, uid={self.uid}, name={self.name}, group={self.group_name}, "
            f"lan_ip={self.lan_ip}, wan_ip={self.wan_ip}, start_time={self.start_time}, "
            f"end_time={self.end_time}, run_time={self.run_time}, log_file={self.log_file}, "
            f"file={self.file}, interpreter={self.interpreter}, status={self.status}, "
            f"insert_count={self.insert_count}, progress={self.progress}, "
            f"total_progress={self.total_progress}, exception={self.exception})")

    @staticmethod
    def up(name, group_name='test', insert_count=0, insert_table='', data=None, uid=None,url='', debug=False) -> Optional[str]:
        if data is None:
            data = {}
        now = datetime.datetime.now()
        if uid is None:
            uid = str(uuid.uuid4())
            data['name'] = name
            data['startTime'] = now.strftime("%Y-%m-%d %H:%M:%S")
            data['groupName'] = group_name
            data['insertTable'] = insert_table
            data['url'] = url
        data['uid'] = uid
        data['insertCount'] = insert_count
        data['status'] = 2
        data['endTime'] = now.strftime("%Y-%m-%d %H:%M:%S")
        try:
            response: dict = requests.post(url=PY_MONITOR_ENDPOINT, json=data, timeout=4).json()
            if debug:
                logger.debug(response)
        except Exception as e:
            if debug:
                logger.error(e)
            return None
        return uid
